/*
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.inteligr8.activiti.doclet;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.TypeElement;

/**
 * @author brian@inteligr8.com
 */
class ActivitiApiBeanDoc {
	
	private final String beanId;
	private TypeElement classElement;
	private boolean isDelegate = false;
	private boolean isExecutionListener = false;
	private boolean isTaskListener = false;
	private Map<String, ExecutableElement> methodElements = new HashMap<>();
	
	public ActivitiApiBeanDoc(String beanId, TypeElement classElement) {
		this.beanId = beanId;
		this.classElement = classElement;
	}
	
	public String getBeanId() {
		return beanId;
	}
	
	public TypeElement getClassElement() {
		return classElement;
	}
	
	public ActivitiApiBeanDoc addMethod(ExecutableElement method) {
		this.methodElements.put(method.toString(), method);
		return this;
	}
	
	public List<ExecutableElement> getSortedMethodElements() {
		List<ExecutableElement> methodElements = new ArrayList<>(this.methodElements.size());
		methodElements.addAll(this.methodElements.values());
		Collections.sort(methodElements, new Comparator<ExecutableElement>() {
			@Override
			public int compare(ExecutableElement methodElement1, ExecutableElement methodElement2) {
				// TODO sort number parts of methods intelligently: method2 before method10
				return methodElement1.getSimpleName().toString().compareToIgnoreCase(methodElement2.getSimpleName().toString());
			}
		});
		
		return methodElements;
	}
	
	public boolean isDelegate() {
		return isDelegate;
	}
	
	public void setDelegate(boolean isDelegate) {
		this.isDelegate = isDelegate;
	}
	
	public boolean isExecutionListener() {
		return isExecutionListener;
	}
	
	public void setExecutionListener(boolean isExecutionListener) {
		this.isExecutionListener = isExecutionListener;
	}
	
	public boolean isTaskListener() {
		return isTaskListener;
	}
	
	public void setTaskListener(boolean isTaskListener) {
		this.isTaskListener = isTaskListener;
	}
	
	@Override
	public String toString() {
		return this.beanId;
	}

}
